import { default as EventEmitter } from 'eventemitter3';
import { EventType } from "../EventType";
import ElementInterface from '../ElementInterface';
import { ValueType } from './ValueType';
export default abstract class AbstractElement implements ElementInterface {
    protected readonly eventEmitter: EventEmitter<EventType, any>;
    protected readonly htmlContainer: HTMLElement;
    private label;
    private elementValue;
    constructor();
    /** the current value of the element */
    get value(): ValueType;
    /** returns all the (input) elements to append to the html-container */
    protected abstract getElementsToAppend(): HTMLElement[];
    /** propagates this elements value to all the inputs included in this element */
    protected abstract propagateValue(value: ValueType): void;
    getHtmlElement(): HTMLElement;
    setValue(value: ValueType): void;
    /** sets the label to the passed value */
    setLabel(label: string): this;
    /**
     * allows to set a callback to be called on every change. based on the return value of the callback this element
     * will be shown (when true) or hidden (when false).
     */
    showOn(callback: () => boolean): this;
    /** @internal */
    on<K extends keyof EventType>(eventname: K, callback: EventEmitter.EventListener<EventType, K>): void;
    /** @internal */
    reportSettingsChange(): void;
    protected createLabel(): HTMLElement | undefined;
    /** to be called, when the internal input value has changed */
    protected changeValue(value: string): void;
    private fillHtmlContainer;
}
