import { default as EventEmitter } from 'eventemitter3';
export default class AbstractElement {
    constructor() {
        this.eventEmitter = new EventEmitter();
        this.label = '';
        this.elementValue = '';
        this.htmlContainer = document.createElement('div');
    }
    /** the current value of the element */
    get value() {
        return this.elementValue;
    }
    getHtmlElement() {
        if (this.htmlContainer.childElementCount === 0) {
            this.fillHtmlContainer();
        }
        return this.htmlContainer;
    }
    setValue(value) {
        this.elementValue = value;
        this.propagateValue(value);
    }
    /** sets the label to the passed value */
    setLabel(label) {
        this.label = label;
        return this;
    }
    /**
     * allows to set a callback to be called on every change. based on the return value of the callback this element
     * will be shown (when true) or hidden (when false).
     */
    showOn(callback) {
        const setDisplay = () => (this.htmlContainer.style.display = callback() ? 'flex' : 'none');
        this.eventEmitter.on('change-settings', setDisplay);
        setDisplay();
        return this;
    }
    /** @internal */
    on(eventname, callback) {
        this.eventEmitter.on(eventname, callback);
    }
    /** @internal */
    reportSettingsChange() {
        this.eventEmitter.emit('change-settings');
    }
    createLabel() {
        if (!this.label) {
            return;
        }
        const label = document.createElement('div');
        label.classList.add('sdpi-item-label');
        label.textContent = this.label;
        return label;
    }
    /** to be called, when the internal input value has changed */
    changeValue(value) {
        this.setValue(value);
        this.eventEmitter.emit('change-value');
    }
    fillHtmlContainer() {
        this.htmlContainer.classList.add('sdpi-item');
        const label = this.createLabel();
        if (label) {
            this.htmlContainer.append(label);
        }
        for (const element of this.getElementsToAppend()) {
            this.htmlContainer.append(element);
        }
    }
}
